@@ -14,8 +14,8 @@ list2lang = function(x) {
     stopf("'x' must be a list")
   if (is.AsIs(x))
     return(rm.AsIs(x))
-  asis = vapply(x, is.AsIs, FALSE)
-  char = vapply(x, is.character, FALSE)
+  asis = vapply_1b(x, is.AsIs)
+  char = vapply_1b(x, is.character)
   to.name = !asis & char
   if (any(to.name)) { ## turns "my_name" character scalar into `my_name` symbol, for convenience
     if (any(non.scalar.char <- lengths(x[to.name])!=1L)) {
@@ -24,7 +24,7 @@ list2lang = function(x) {
     x[to.name] = lapply(x[to.name], as.name)
   }
   if (isTRUE(getOption("datatable.enlist", TRUE))) { ## recursively enlist for nested lists, see note section in substitute2 manual
-    islt = vapply(x, only.list, FALSE) #5057 nested DT that inherits from a list must not be turned into list call
+    islt = vapply_1b(x, only.list) #5057 nested DT that inherits from a list must not be turned into list call
     to.enlist = !asis & islt
     if (any(to.enlist)) {
       x[to.enlist] = lapply(x[to.enlist], enlist)
